#!/usr/bin/env node
import args from 'yargs';
import chalk from 'chalk';
import solc from 'solc';
import fs from 'fs-extra';
import path from 'path';

/**
 * Convert Array to Object
 *
 * @param  {Array}  a Array to convert
 * @return {Object}   Converted array
 */
const arrayToObject = a => a.reduce((o, v) => Object.assign({}, o, v), {});

/**
 * Compile a set of one or more contracts
 *
 * @example compileContracts({ 'Echo.sol': 'pragma solidity ^0.4.15; contract Echo {}' })
 *
 * @param  {Object} sources Stringified contracts to compile
 * @return {Object}         Compiled contracts' ABI and bytecode output
 */
const compileContracts = sources => {
  // compile contracts
  const output = solc.compile({ sources }, 1);

  // get compiler output
  const { contracts, errors } = output;

  // catch compiler errors
  if (errors) {
    throw new Error(errors[0].message);
  }

  // get `ABI` and `bytecode` from the compiled contract, keyed by contract name
  return arrayToObject(Object.keys(contracts).map(contract => ({
    [`${contract.split(':')[1]}`]: {
      abi: JSON.parse(contracts[contract].interface),
      bytecode: contracts[contract].bytecode
    }
  })));
};

/**
 * Extract contents of contract files as string
 *
 * @param  {Array}  contracts             Contracts to compile
 * @param  {String} inputPath             Directory to read target contracts from
 * @param  {String} contractFileExtension File extension of the target contracts
 * @return {Array}                        Promises resolving contract file content
 */
const getContractSources = (contracts, inputPath, contractFileExtension) => {
  return contracts.map(async contract => {
    try {
      // read contract file content
      const input = await fs.readFileSync(path.resolve(`${inputPath}/${contract}.${contractFileExtension}`), 'utf8');

      // return key/value of contractName.contractFileExtension and file content
      return { [`${contract}.${contractFileExtension}`]: input };
    } catch (error) {
      throw new Error(error);
    }
  });
};

/**
 * Default configuration values
 *
 * @type {Object}
 */
const DEFAULT_CONFIG = {
  contractFileExtension: 'sol',
  outputFilename: 'contracts'

  /**
   * Process configuration options for the compilation process
   *
   * @param  {Object} options Raw configuration options
   * @return {Object}         Processed configuration options
   */
};const processOptions = options => {
  // import config options with `config` filepath argument
  const config = require(path.resolve(options.config));

  // if `outputPath` is undefined use `inputPath`
  if (!config.outputPath) config.outputPath = config.inputPath;

  // merge provided config with default config
  return Object.assign({}, DEFAULT_CONFIG, config);
};

/**
 * Write compiled contracts interfaces to disk
 *
 * @param  {Object} contracts      Compiled contracts
 * @param  {Object} outputFilename File to write compiled contracts to
 * @param  {Object} outputPath     Directory to write compiled contracts to
 * @return {Promise}
 */
const outputCompiledContracts = (contracts, outputFilename, outputPath) => {
  return fs.outputJson(`${outputPath}/${outputFilename}.json`, contracts, {
    spaces: 2
  });
};

(async () => {
  try {
    const {
      contractFileExtension,
      contracts,
      inputPath,
      outputFilename,
      outputPath
    } = processOptions(args.argv);

    // get the stringified content of all contracts
    const contractSources = await Promise.all(getContractSources(contracts, inputPath, contractFileExtension));

    // do what we came here to do
    const compiledContracts = compileContracts(arrayToObject(contractSources));

    // output compiled contract interfaces
    await outputCompiledContracts(compiledContracts, outputFilename, outputPath);

    // all done
    return console.log(chalk.green.bold('\n😂‍   Contracts compiled successfully'));
  } catch (error) {
    throw new Error(error);
  }
})();
//# sourceMappingURL=sulk.mjs.map
